#include <amxmodx>
#include <easy_http>
#include <nvault>

#define PLUGIN		"Telegram Reports Sender"
#define VERSION		"3.0.1"
#define AUTHOR		"@pywave & Albertio"
#define DICTIONARY	"telegram_reports_sender.txt"
#define CFG_FILE	"addons/amxmodx/configs/plugins/telegram_reports_sender.ini"

enum {
	SECTION_NONE = -1,
	SECTION_SETTINGS
};

new g_iCurrentSection;

enum _:PluginSettings {
	SEND_MESSAGE[128],
	SEND_PHOTO[128],
	PHOTO_URL[128],
	REPORT_DELAY
};

new g_ePluginSettings[PluginSettings];

enum _:ChatData {
	CHAT_ID[32],
	PUNISHMENT_THREAD_ID,
	REPORT_THREAD_ID
};

new Array:g_aTelegramChats;
new g_iTelegramChatsNum;

new Array:g_aCommands;
new g_iCommandsNum;

enum _:HostData {
	HostMapName[64],
	HostName[64]
};

new g_eHostData[HostData];

new g_hVault;

// Fresh Bans
forward fbans_player_banned_pre_f(
	const id,
	const uid, 
	const player_steamid[],
	const player_ip[],
	const player_name[], 
	const admin_ip[],
	const admin_steamid[],
	const admin_name[], 
	const ban_type[],
	const ban_reason[],
	const bantime
);

// [fork] Lite Bans
forward user_banned_pre(id, admin_id, ban_minutes);

// AMXBans RBS
forward amxbans_ban_pre(
	id, 
	admin, 
	bantime, 
	bantype[], 
	banreason[]
);

// Chat Additions: Gag
forward CA_gag_setted(
	const target,
	name[],
	authID[],
	IP[],
	adminName[],
	adminAuthID[],
	adminIP[],
	reason[],
	time,
	gag_flags_s:flags,
	expireAt
);

// Ultimate GAG
forward gag_gaged(id, player, flags, unixtime, reason[]);

public plugin_precache() {
	g_aTelegramChats = ArrayCreate(ChatData);
	g_aCommands = ArrayCreate(32);

	new INIParser:iParser = INI_CreateParser();
	INI_SetReaders(iParser, "ReadKeyValue", "ReadNewSection");
	INI_ParseFile(iParser, CFG_FILE);
	INI_DestroyParser(iParser);

	g_iTelegramChatsNum = ArraySize(g_aTelegramChats);
	g_iCommandsNum = ArraySize(g_aCommands);

	if(!g_iTelegramChatsNum)
		set_fail_state("Specify the Telegram chat id!");

	if(g_ePluginSettings[SEND_MESSAGE][0] == EOS)
		set_fail_state("Specify the Telegram bot token!");
}

public plugin_init() {
	register_plugin(PLUGIN, VERSION, AUTHOR);

	register_dictionary(DICTIONARY);

	register_clcmd("say", "SayReport");
	register_clcmd("say_team", "SayReport");

	get_mapname(g_eHostData[HostMapName], charsmax(g_eHostData[HostMapName]));

	bind_pcvar_string(get_cvar_pointer("hostname"), g_eHostData[HostName], charsmax(g_eHostData[HostName]));

	formatex(g_ePluginSettings[PHOTO_URL], charsmax(g_ePluginSettings[PHOTO_URL]),
	"https://image.gametracker.com/images/maps/160x120/cs/%s.jpg", g_eHostData[HostMapName]);
}

public plugin_cfg()	{
	if((g_hVault = nvault_open("telegram_reports_sender")) == INVALID_HANDLE)
		set_fail_state("Error opening nVault!");

	nvault_prune(g_hVault, 0, get_systime() - g_ePluginSettings[REPORT_DELAY]);
}

public plugin_end() {
	nvault_close(g_hVault);
}

// Fresh Bans
public fbans_player_banned_pre_f(
	const id,
	const userid,
	const szSteamID[],
	const szIp[],
	const szName[],
	const szAdminIp[],
	const szAdminSteamID[],
	const szAdminName[],
	const ban_type[],
	const szReason[],
	const bantime
) {
	new szTime[12];
	num_to_str(bantime, szTime, charsmax(szTime));

	SendPunishmentMessage(fmt("%L", LANG_SERVER, "BAN"), szSteamID, szIp, szName, szAdminSteamID, szAdminIp, szAdminName, szReason, szTime);
}

// [fork] Lite Bans
public user_banned_pre(id, admin_id, ban_minutes) {
	new szName[MAX_NAME_LENGTH], szAdminName[MAX_NAME_LENGTH];
	get_user_name(id, szName, charsmax(szName));
	get_user_name(admin_id, szAdminName, charsmax(szAdminName));

	new szIp[MAX_IP_WITH_PORT_LENGTH], szAdminIp[MAX_IP_WITH_PORT_LENGTH];
	get_user_ip(id, szIp, charsmax(szIp), 1);
	get_user_ip(admin_id, szAdminIp, charsmax(szAdminIp), 1);

	new szAuthID[MAX_AUTHID_LENGTH], szAdminAuthID[MAX_AUTHID_LENGTH];
	get_user_authid(id, szAuthID, charsmax(szAuthID));
	get_user_authid(admin_id, szAdminAuthID, charsmax(szAdminAuthID));

	new szTime[12];
	num_to_str(ban_minutes, szTime, charsmax(szTime));

	SendPunishmentMessage(fmt("%L", LANG_SERVER, "BAN"), szAuthID, szIp, szName, szAdminAuthID, szAdminIp, szAdminName, "", szTime);
}

// AMXBans RBS
public amxbans_ban_pre(
	id, 
	admin, 
	bantime, 
	bantype[], 
	banreason[]
) {
	new szName[MAX_NAME_LENGTH], szAdminName[MAX_NAME_LENGTH];
	get_user_name(id, szName, charsmax(szName));
	get_user_name(admin, szAdminName, charsmax(szAdminName));

	new szIp[MAX_IP_WITH_PORT_LENGTH], szAdminIp[MAX_IP_WITH_PORT_LENGTH];
	get_user_ip(id, szIp, charsmax(szIp), 1);
	get_user_ip(admin, szAdminIp, charsmax(szAdminIp), 1);

	new szAuthID[MAX_AUTHID_LENGTH], szAdminAuthID[MAX_AUTHID_LENGTH];
	get_user_authid(id, szAuthID, charsmax(szAuthID));
	get_user_authid(admin, szAdminAuthID, charsmax(szAdminAuthID));

	new szTime[12];
	num_to_str(bantime, szTime, charsmax(szTime));

	SendPunishmentMessage(fmt("%L", LANG_SERVER, "BAN"), szAuthID, szIp, szName, szAdminAuthID, szAdminIp, szAdminName, banreason, szTime);
}

// Chat Additions: Gag
public CA_gag_setted(
	const target,
	szName[],
	authID[],
	IP[],
	adminName[],
	adminAuthID[],
	adminIP[],
	szReason[],
	iTime,
	gag_flags_s:flags,
	expireAt
) {
	new szTime[12];
	num_to_str(iTime / 60, szTime, charsmax(szTime));

	SendPunishmentMessage(fmt("%L", LANG_SERVER, "MUTE"), authID, IP, szName, adminAuthID, adminIP, adminName, szReason, szTime);
}

// Ultimate GAG
public gag_gaged(id, player, flags, unixtime, reason[]) {
	new szName[MAX_NAME_LENGTH], szAdminName[MAX_NAME_LENGTH];
	get_user_name(player, szName, charsmax(szName));
	get_user_name(id, szAdminName, charsmax(szAdminName));

	new szIp[MAX_IP_WITH_PORT_LENGTH], szAdminIp[MAX_IP_WITH_PORT_LENGTH];
	get_user_ip(player, szIp, charsmax(szIp), 1);
	get_user_ip(id, szAdminIp, charsmax(szAdminIp), 1);

	new szAuthID[MAX_AUTHID_LENGTH], szAdminAuthID[MAX_AUTHID_LENGTH];
	get_user_authid(player, szAuthID, charsmax(szAuthID));
	get_user_authid(id, szAdminAuthID, charsmax(szAdminAuthID));

	new szTime[12];
	num_to_str((unixtime - get_systime()) / 60, szTime, charsmax(szTime));

	SendPunishmentMessage(fmt("%L", LANG_SERVER, "MUTE"), szAuthID, szIp, szName, szAdminAuthID, szAdminIp, szAdminName, reason, szTime);
}

public SayReport(id) {
	new szCmd[32], szCmdArgs[192];
	new bool:bCmdFound;
	new iCmdLen;

	read_argv(1, szCmdArgs, charsmax(szCmdArgs));

	for(new i; i < g_iCommandsNum; i++) {
		ArrayGetString(g_aCommands, i, szCmd, charsmax(szCmd));
		iCmdLen = strlen(szCmd);

		if(equali(szCmd, szCmdArgs, iCmdLen) && (szCmdArgs[iCmdLen] == EOS || szCmdArgs[iCmdLen] == ' ')) {
			bCmdFound = true;
			break;
		}
	}

	trim(szCmdArgs[iCmdLen + 1]);

	if(!bCmdFound)
		return;

	if(szCmdArgs[iCmdLen + 1] == EOS) {
		client_print_color(id, print_team_red, "%L", id, "ERROR_REPORT");
		return;
	}

	new szAuthID[MAX_AUTHID_LENGTH];
	get_user_authid(id, szAuthID, charsmax(szAuthID));

	new iReportDelay = nvault_get(g_hVault, szAuthID);
	new iSysTime = get_systime();

	if(iReportDelay && iSysTime < iReportDelay) {
		client_print_color(id, print_team_red, "%L", id, "REPORT_DELAY", iReportDelay - iSysTime);
		return;
	}

	new szName[MAX_NAME_LENGTH];
	get_user_name(id, szName, charsmax(szName));

	new szIp[MAX_IP_WITH_PORT_LENGTH];
	get_user_ip(id, szIp, charsmax(szIp), 1);

	SendReportMessage(id, szCmdArgs[iCmdLen + 1], szAuthID, szIp, szName);
}

public SendPunishmentMessage(
	const szPunishment[],
	const szSteamID[],
	const szIp[],
	const szName[],
	const szAdminSteamID[],
	const szAdminIp[],
	const szAdminName[],
	const szReason[],
	const szDuration[]
) {
	new szFmtMessage[1024];
	formatex(szFmtMessage, charsmax(szFmtMessage), "%L", LANG_SERVER, "BAN_MUTE_MESSAGE", g_eHostData[HostName], g_eHostData[HostMapName],
	szName, szIp, szSteamID, szAdminName, szAdminIp, szAdminSteamID, szPunishment, szReason, szDuration);

	BuildRequest(0, szFmtMessage, g_ePluginSettings[SEND_PHOTO], g_ePluginSettings[PHOTO_URL]);
}

public SendReportMessage(const id, const szReportMessage[], const szSteamID[], const szIp[], const szName[]) {
	new szFmtMessage[1024];
	formatex(szFmtMessage, charsmax(szFmtMessage), "%L", LANG_SERVER, "REPORT_MESSAGE",
	g_eHostData[HostName], g_eHostData[HostMapName], szName, szIp, szSteamID, szReportMessage);

	BuildRequest(id, szFmtMessage, g_ePluginSettings[SEND_MESSAGE]);
}

public TelegramSendMessage(EzHttpRequest:eRequestID) {
	if(ezhttp_get_error_code(eRequestID) != EZH_OK) {
		new szError[64];
		ezhttp_get_error_message(eRequestID, szError, charsmax(szError));
		set_fail_state("%L", LANG_SERVER, "ERROR_RESPONSE", szError);
		return;
	}

	new iData[2], id, iChatIndex;
	ezhttp_get_user_data(eRequestID, iData);
	id = iData[0];
	iChatIndex = iData[1];

	if(is_user_connected(id)) {
		new szAuthID[MAX_AUTHID_LENGTH];
		get_user_authid(id, szAuthID, charsmax(szAuthID));

		new szReportDelay[32];
		num_to_str(get_systime() + g_ePluginSettings[REPORT_DELAY], szReportDelay, charsmax(szReportDelay));

		nvault_set(g_hVault, szAuthID, szReportDelay);

		if(iChatIndex == 0)
			client_print_color(id, print_team_blue, "%L", id, "SUCCESSFUL_REPORT");
	}
}

public bool:ReadNewSection(INIParser:iParser, const szSection[], bool:bInvalidTokens, bool:bCloseBracket) {
	if(!bCloseBracket) {
		log_amx("Closing bracket was not detected! Current section name '%s'.", szSection);
		return false;
	}

	if(equal(szSection, "SETTINGS")) {
		g_iCurrentSection = SECTION_SETTINGS;
		return true;
	}

	return false;
}

public bool:ReadKeyValue(INIParser:iParser, const szKey[], const szValue[]) {
	if(szKey[0] == EOS || szValue[0] == EOS)
		return false;

	switch(g_iCurrentSection) {
		case SECTION_NONE: { return false; }
		case SECTION_SETTINGS: {
			if(equal(szKey, "BOT_TOKEN")) {
				formatex(g_ePluginSettings[SEND_MESSAGE], charsmax(g_ePluginSettings[SEND_MESSAGE]), "https://api.telegram.org/bot%s/sendMessage?", szValue);
				formatex(g_ePluginSettings[SEND_PHOTO], charsmax(g_ePluginSettings[SEND_PHOTO]), "https://api.telegram.org/bot%s/sendPhoto?", szValue);
			} else if(equal(szKey, "CHATS_IDS")) {
				new szTgChat[32], szMsgThread[2][32], eData[ChatData], szAllTgChats[MAX_FMT_LENGTH];
				copy(szAllTgChats, charsmax(szAllTgChats), szValue);

				while(szAllTgChats[0] != 0 && strtok(szAllTgChats, szTgChat, charsmax(szTgChat), szAllTgChats, charsmax(szAllTgChats), ',')) {
					trim(szTgChat), trim(szAllTgChats);
					strtok(szTgChat, eData[CHAT_ID], charsmax(eData[CHAT_ID]), szMsgThread[0], charsmax(szMsgThread[]), ':');

					if(szMsgThread[0][0] != EOS) {
						trim(szTgChat);
						strtok(szMsgThread[0], szMsgThread[0], charsmax(szMsgThread[]), szMsgThread[1], charsmax(szMsgThread[]), ':');
						eData[PUNISHMENT_THREAD_ID] = str_to_num(szMsgThread[0]);
						eData[REPORT_THREAD_ID] = str_to_num(szMsgThread[1]);
					} else {
						eData[PUNISHMENT_THREAD_ID] = INVALID_HANDLE;
						eData[REPORT_THREAD_ID] = INVALID_HANDLE;
					}

					ArrayPushArray(g_aTelegramChats, eData);
				}
			} else if(equal(szKey, "REPORT_COMMANDS")) {
				new szCmd[32], szAllCmds[MAX_FMT_LENGTH];
				copy(szAllCmds, charsmax(szAllCmds), szValue);

				while(szAllCmds[0] != 0 && strtok(szAllCmds, szCmd, charsmax(szCmd), szAllCmds, charsmax(szAllCmds), ',')) {
					trim(szCmd), trim(szAllCmds);
					ArrayPushString(g_aCommands, szCmd);
				}
			} else if(equal(szKey, "REPORT_DELAY")) { g_ePluginSettings[REPORT_DELAY] = str_to_num(szValue); }
		}
	}

	return true;
}

stock BuildRequest(const id, const szText[], const szUrl[], const szPhotoUrl[] = "") {
	for(new iChatIndex, eData[ChatData]; iChatIndex < g_iTelegramChatsNum; iChatIndex++) {
		ArrayGetArray(g_aTelegramChats, iChatIndex, eData);

		new EzJSON:eObject = ezjson_init_object();

		if(eObject == EzInvalid_JSON)
			continue;

		ezjson_object_set_string(eObject, "chat_id", eData[CHAT_ID]);
		ezjson_object_set_string(eObject, "parse_mode", "HTML");

		if(szPhotoUrl[0] != EOS) {
			ezjson_object_set_string(eObject, "photo", szPhotoUrl);
			ezjson_object_set_string(eObject, "caption", szText);

			if(eData[PUNISHMENT_THREAD_ID] > INVALID_HANDLE)
				ezjson_object_set_number(eObject, "message_thread_id", eData[PUNISHMENT_THREAD_ID]);
		} else {
			ezjson_object_set_string(eObject, "text", szText);

			if(eData[REPORT_THREAD_ID] > INVALID_HANDLE)
				ezjson_object_set_number(eObject, "message_thread_id", eData[REPORT_THREAD_ID]);
		}

		new EzHttpOptions:eOptions = ezhttp_create_options();

		ezhttp_option_set_body_from_json(eOptions, eObject);
		ezjson_free(eObject);
		ezhttp_option_set_header(eOptions, "Content-Type", "application/json");

		if(id) {
			new iData[2];
			iData[0] = id;
			iData[1] = iChatIndex;
			ezhttp_option_set_user_data(eOptions, iData, sizeof(iData));
		}

		ezhttp_post(szUrl, "TelegramSendMessage", eOptions);
	}
}